%% - Uav model, defect vector calculation%% 
%
% x0 / x6  / x(0+i*6) : North pos. coordinate [m]
% x1 / x7  / x(1+i*6) : East pos. coordinate [m]
% x2 / x8  / x(2+i*6) : Uav speed [m/s]
% x3 / x9  / x(3+i*6) : Uav heading [radian]
%
% x4 / x10 / x(4+i*6) : Uav Input 1 - Acceleration [m/s^2]
% x5 / x11 / x(5+i*6) : Uav Input 2 - Bank angle [radians]

disp('Calculating defect vector')

% Modll / defect%
x_l=[ x0; x1; x2; x3;];  % state variables, left node
u_l=[ x4; x5;];                  % inputs, left node

x_r=[ x6; x7;x8;x9;];  % state variables, right node
u_r=[x10;x11;];                  % inputs, right node


% MODEL: system eq., left node                     
fx_l = [    x2*cos(x3);         % x0_dot=..., x0 = north 
            x2*sin(x3);         % x1_dot=..., x1 = east
            x4;                 % x2_dot=..., x2 = speed
            grav*tan(x5)/x2;];  % x3_dot=..., x3 = heading
            
    
fx_r = [    x8*cos(x9);     % MODEL: system eq., right node
            x8*sin(x9);
            x10;
            grav*tan(x11)/x8;];
        
x_c = (x_l+x_r)/2 + T*(fx_l-fx_r)/8;    % state vector at collocation point
u_c = (u_l + u_r )/2;                   % input vector at collocation point

x_c_dot = (-3)*(x_l-x_r)/(2*T) - (fx_l+fx_r)/4 ;

fx_c = [    x_c(3)*cos(x_c(4));         % system eq. in collocation point
            x_c(3)*sin(x_c(4));         % note: 1-indexing in matlab!
            u_c(1);
            grav*tan(u_c(2))/x_c(3);];

g = fx_c - x_c_dot; % calculating defect vector. 
                    % defects = constraints, called g[] in IPOPT
