%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                        %
% Problem setup file for optimization with                               %
% Collocation of UAV model only                                          %
% Quadratic Distance minimizing                                          %
%                                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Following must be in place: 
% - problem parameters in this file
% - Correct system model in UAV_q_1_ConstraintSetup.m
% - Correct system model in methods RK4: SV and 
% - Structure of Overlaping blocks in Hessian, UAV_q_1_ConstraintSetup.m 
clc;
clear all;

%% - Symbols and misc parameters

syms x0 x1 x2 x3 x4 x5 x6 x7 x8 x9 x10 x11
syms lambda0 lambda1 lambda2 lambda3 lambda4 lambda5  
syms lambda6 lambda7 
syms symfix obj_factor 
syms T grav 


% state vector, first segment
x = [x0;x1;x2;x3;x4;x5;x6;x7;x8;x9;x10;x11;]; 

% PROBLEM PARAMETERES TO HARDCODE %
segments = 60;
svn      = 4;            % SVN : System variables pr node (n)
svn_str  = int2str(svn);
ovn      = 6;            % OVN : Optimization variables pr node (n+m)
ovn_str  = num2str(ovn);

% Paths to textfile wich is written to by script, included in C++ progr.
textfiles_path = 'C:\Users\grimslan\Dropbox\ShortFlight - Simple\textfiles\';

g_ccode_path   = strcat(textfiles_path, 'g_ccode.txt');
JacVal_path    = strcat(textfiles_path, 'JacVal.txt');
JacPos_path    = strcat(textfiles_path, 'JacPos.txt');
objective_path = strcat(textfiles_path, 'objective.txt');
obj_jac_path   = strcat(textfiles_path, 'obj_jac.txt');
hesspos_path   = strcat(textfiles_path, 'hesspos.txt');
hessval_path   = strcat(textfiles_path, 'hessval.txt');
hessval2_path  = strcat(textfiles_path, 'hessval2.txt');
startval_path  = strcat(textfiles_path, 'startval.txt');

% Block related constants are automaticly updated later on
jnb     = -1;           % JNB : Jacobian nonzeros pr block
jnb_str = num2str(jnb);
hnb     = -1;           % HNB : Hessian nonzeros first block
hnb_str = num2str(hnb);
hnba    = -1;           % HNBA: Hessian nonzeros added by block 2,3,...
hnba_str= num2str(hnba);


%% - Optimization start point - %%
UAV_q_1_StartPoint;

%% - Uav model, COLLOCATION defect vector / constraints calculated - %% 
UAV_q_1_ConstraintSetup;
                    
%% - Writing constraints to file (as C code)-  %%
UAV_q_1_ConstraintWrite;

%% - Constraints Jacobian - %%
UAV_q_1_ConstraintsJacobian;

%% - Objective Functions, Eval + gradient - %%
UAV_q_1_ObjectiveFcn;

%% - Gradient of objective function - %% 
UAV_q_1_ObjectiveGradient;

%% Hessian of Lagrange Function %%
UAV_q_1_Hessian