%% - Objective Function setup - %%

%% - Defining Objective - %% 

% wheighting parameters
syms Wa Wr Wt Wu0 Wu1 Wu2 Wu3;     
% targte positions (grid centers)
syms T0N T0E T1N T1E T2N T2E T3N T3E;
syms k0 k1 k2 k3 

% Objective Function for first block (IPOPT minimizes objective)
% starting from second node since most variables in first node are fixed
% Comment out unwanted functions or targets

% Quadratic objective %
%objective =  + Wa*(x10)^2 + Wr*(x11)^2 + Wu0*((T0N-x6)^2+(T0E-x7)^2) + Wu1*((T1N-x6)^2+(T1E-x7)^2) + Wu2*((T2N-x6)^2+(T2E-x7)^2) + Wu3*((T3N-x6)^2+(T3E-x7)^2) ; 
% Norm Objective %
objective =  + Wa*(x10)^2 + Wr*(x11)^2 + Wu0*sqrt((T0N-x6)^2+(T0E-x7)^2) + Wu1*sqrt((T1N-x6)^2+(T1E-x7)^2) + Wu2*sqrt((T2N-x6)^2+(T2E-x7)^2) + Wu3*sqrt((T3N-x6)^2+(T3E-x7)^2) ; 
% Exponential objective %
%objective =  Wa*(x10)^2 + Wr*(x11)^2 -Wu0*exp(-k0*sqrt((T0N-x6)^2+(T0E-x7)^2))% - Wu1*exp(-k1*sqrt((T1N-x6)^2+(T1E-x7)^2))% - Wu2*(-k2*sqrt((T2N-x6)^2+(T2E-x7)^2)) - Wu3*exp(-k3*sqrt((T3N-x6)^2+(T3E-x7)^2)) ; 


disp('Objective Function initialized')

%% - Translating to generalized C-string - %%
objectiveStringC = ccode(objective);
% removing unnecessary name
objectiveStringC = strrep(objectiveStringC, 't0 = ', '');
% removing unnecessary ;
objectiveStringC = strrep(objectiveStringC, ';', '');     

objectiveToWrite = ''; 

for i=0:segments-1 % variable index increase by OVN pr block
    incr = num2str(i*ovn); % 0,6,12,... OVN - optimiz variables pr node 
    % correcting variable names/indexes to C-style.
    % All variable indices increases with (m+n) for each block
    modifiedStr  = objectiveStringC;    
    newVar1= strcat( 'x[6+', int2str(i*ovn), ']' );
    modifiedStr  = strrep(modifiedStr, 'x6', newVar1 );
    newVar2= strcat( 'x[7+', int2str(i*ovn), ']' );
    modifiedStr  = strrep(modifiedStr, 'x7', newVar2 );
    newVar3= strcat( 'x[10+', int2str(i*ovn), ']' );
    modifiedStr  = strrep(modifiedStr, 'x10', newVar3 );
    newVar4= strcat( 'x[11+', int2str(i*ovn), ']' );
    modifiedStr  = strrep(modifiedStr, 'x11', newVar4 );
   
    % Note! if more variables are added to objective - add more var-replace
    
    
    if  i ~= segments-1  % unless last segment, add a +
        objectiveToWrite = strcat(objectiveToWrite, modifiedStr, ' + ');
    else
        objectiveToWrite = strcat(objectiveToWrite, modifiedStr );
    end
end 

% adding variable declaration and end of line, as c-syntax rewuires
objectiveToWrite  = strcat( 'obj_value = ' , objectiveToWrite, ';' );

disp('Objective Function generaliced, parsed to C-Code')

%% - Write to file - %%
% writing objective function string to text file, ready to be included in
% the IPOPT C++ framework
% Object fcn string for ALL segments. No loop/editing in C app.

fid = fopen(objective_path, 'w');
fclose(fid);
fid = fopen(objective_path, 'at');
fprintf(fid, objectiveToWrite);
fclose(fid);

disp('Objective Function Written to file')