%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                        %
%   UAV_q_1_PlotSim.m                                                    %
%   Reads results from IPOPT program from text file,                     %
%   organizes, plots and simulates result with a RK4                     %
%                                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc;

%% Reading various PARMATETERS from optimization %%
parameter_id = fopen('C:\Users\grimslan\Dropbox\ShortFlight - Simple\Release\parameters.txt', 'r');
% Problem data %
tline = fgetl(parameter_id)
model_states = str2double(tline)
tline        = fgetl(parameter_id);
model_inputs = str2double(tline)
tline        = fgetl(parameter_id);
T            = str2double(tline)
tline        = fgetl(parameter_id);
recalcNodeNo = str2double(tline)
	
% Uncertainty model 0 parameteres %
tline  = fgetl(parameter_id);
alpha0 = str2double(tline)    
tline  = fgetl(parameter_id);
b0     = str2double(tline)
tline  = fgetl(parameter_id);
k0     = str2double(tline)
tline  = fgetl(parameter_id);
ceta0  = str2double(tline)
tline  = fgetl(parameter_id);
T0n    = str2double(tline)
tline  = fgetl(parameter_id);
T0e    = str2double(tline)

% Uncertainty model 1 parameteres %
tline = fgetl(parameter_id)
alpha1 = str2double(tline)
tline = fgetl(parameter_id)
b1 = str2double(tline)
tline = fgetl(parameter_id)
k1 = str2double(tline)
tline = fgetl(parameter_id)
ceta1 = str2double(tline)
tline = fgetl(parameter_id)
T1n = str2double(tline)
tline = fgetl(parameter_id)
T1e = str2double(tline)
	
% Uncertainty model 2 parameteres %
tline = fgetl(parameter_id)
alpha2 = str2double(tline)
tline = fgetl(parameter_id)
b2 = str2double(tline)
tline = fgetl(parameter_id)
k1 = str2double(tline)
tline = fgetl(parameter_id)
ceta2 = str2double(tline)
tline = fgetl(parameter_id)
T2n = str2double(tline)
tline = fgetl(parameter_id)
T2e = str2double(tline)

% Uncertainty model 3 parameteres %
tline = fgetl(parameter_id)
alpha3 = str2double(tline)
tline = fgetl(parameter_id)
b3 = str2double(tline)
tline = fgetl(parameter_id)
k3 = str2double(tline)
tline = fgetl(parameter_id)
ceta3 = str2double(tline)
tline = fgetl(parameter_id)
T3n = str2double(tline)
tline = fgetl(parameter_id)
T3e = str2double(tline)

% Objective function parametres %
tline = fgetl(parameter_id)
Wa = str2double(tline)
tline = fgetl(parameter_id)
Wr = str2double(tline)
tline = fgetl(parameter_id)
Wu0 = str2double(tline)
tline = fgetl(parameter_id)
Wu1 = str2double(tline)
tline = fgetl(parameter_id)
Wu2 = str2double(tline)
tline = fgetl(parameter_id)
Wu3 = str2double(tline)
tline = fgetl(parameter_id)
Wt = str2double(tline)


fclose(parameter_id);


%% Reading optimization Result from file, sorting them %%

% _Not_ preallocating to make it as simple as possible
north_c      = [];    % Array of UAV North position coordinates
east_c       = [];    % Array of UAV East position coordinates
uav_speed    = [];    % Array of UAV air speeds
uav_heading  = [];    % Array of UAV headings

acceleration = [];    % Array of UAV input acceleration
bank_angle   = [];    % Array of UAV input banking angle
counter      = 0;

example_id = fopen('C:\Users\grimslan\Dropbox\ShortFlight - Simple\Release\optimal_solution.txt', 'r');
while 1
    tline = fgetl(example_id)
    if ~ischar(tline), break, end
        x = str2double(tline)
        counter = counter+1;
        index = mod(counter,6) % (m+n), opt vars pr node hardcoded
        if (index==1)
            north_c = [north_c;x;];
        elseif (index==2)
            east_c = [east_c;x;];
        elseif (index==3)
            uav_speed = [uav_speed; x;];
        elseif (index==4)
            uav_heading = [uav_heading; x;];
        elseif (index==5)
            acceleration = [acceleration;x;];
        elseif (index==0)
            bank_angle = [bank_angle;x;];
        else
                disp('unexpected occurance while sorting results')
        end
            
end    
fclose(example_id);

%% Plotting the sorted results %%
temp=size(north_c);
N=temp(1);  % N: number of nodes
flytime=(N-1)*T;
timescaler=0:T:(N-1)*T ;


%Plot the optimal path
figure(2)
    plot(east_c,north_c, 'b-o', 'LineWidth', 2);
    title(['Optimal Path w/' num2str(N-1),' segments and ', num2str(flytime), ' sec flight time.']) ;
    xlabel('Metres East, [m]')
    grid on
    hold on
    % comment out unused targets%
    plot(T0e, T0n, 'r-*', 'LineWidth', 10);
    plot(T1e, T1n, 'r-*', 'LineWidth', 10);
    plot(T2e, T2n, 'r-*', 'LineWidth', 10);
    plot(T3e, T3n, 'r-*', 'LineWidth', 10);
    legend('UAV path','Target', 'Location', 'SouthEast')
    hold off
    
% Plot other optimized variables 

%convert uav headning to degrees
uav_heading_plot=uav_heading*180/pi;

figure( 3)    
    subplot(2,1,1),plot(timescaler, uav_heading_plot, 'b-o','LineWidth', 2),title('Heading; (0 degrees = North, 90 degrees = East- )')
        xlabel('Time, [s]')
        ylabel('Heading, [degrees]')
        grid on
    subplot(2,1,2),plot(timescaler,uav_speed, 'b-o', 'LineWidth', 2),title('Groundspeed')
        xlabel('Time, [s]')
        ylabel('Speed, [m/s]')
        grid on
figure(4)    
    subplot(2,1,1)
        stairs(timescaler,acceleration, 'b-o','LineWidth', 2),title('u1 - commanded acceleration')
        xlabel('Time, [s]')
        ylabel('Acceleration, [m/s^2]')
        grid on
    subplot(2,1,2)
        stairs(timescaler,bank_angle, 'b-o','LineWidth', 2), title('u2 - commanded bank angle')
        xlabel('Time, [s]')
        ylabel('Bank angle, [degrees]')
        grid on

%% - SIMULATION of result  - 
startvalues = [ north_c(1);         % UAV North position coordinates
                east_c(1);          % UAV East position coordinates
                uav_speed(1);       % UAV air speeds
                uav_heading(1);     % UAV headings
                acceleration(1);    % UAV input acceleration
                bank_angle(1);  ];  % UAV input banking angle

     
sim_states=UAV_q_1_RK4_FOH(startvalues, acceleration, bank_angle, 0.1*T, T,model_states);

temp=size(sim_states)
tempo=temp(1)
rk_pts=tempo/model_states
sim_north = zeros(rk_pts,1);
sim_east  = zeros(rk_pts,1);

for i=1:1:rk_pts
    sim_north(i) = sim_states(1+model_states*(i-1) );
    sim_east(i)  = sim_states(2+model_states*(i-1) );
    
end

%Plot the optimal path
figure(7)
    plot(east_c,north_c, 'b-o', 'LineWidth', 2), title(['Optimal Path w/' num2str(N-1),' segments and ', num2str(flytime), ' sec flight time']) ;
    xlabel('Metres East, [m]')
    ylabel('Metres North, [m]')
    grid on
    hold on
    plot(sim_east,sim_north, 'r--', 'LineWidth', 2);
    % Comment out targets not used %
    plot(T0e, T0n, 'r-*', 'LineWidth', 10);
    plot(T1e, T1n, 'r-*', 'LineWidth', 10);
   % plot(T2e, T2n, 'r-*', 'LineWidth', 10);
   % plot(T3e, T3n, 'r-*', 'LineWidth', 10);
    legend('Optimal UAV path','Simulated UAV path','Target', 'Location', 'SouthEast')
    hold off