
function y = UAV_q_1_RK4_FOH(startvalues, acceleration, bankangle, steplenght, T, svn)
% y_d = f(t,y)  , y(t_0)=y_0
%initializing y:

disp('Start simulation with RK4 w/FOH on inputs between nodes')

size_arr = size(acceleration);
nodes = size_arr(1);

t = T; % IPOPT segment length in seconds
h = steplenght; % RK4   step    length in seconds 

ratio = t/h;

shift = svn; % number of state variables at each node

%say time steps are 1-indexed here;
y=zeros(shift*(nodes-1)*ratio+shift,1);  % preallocating result array
y(1:shift) = startvalues(1:shift); % cooresponds to y(t0)



rk_step=0;

if ratio == 1 % Steplength CANNOT be greater than t !!
    cut = 0;
else
    cut = 1;
end 

for s=1:1:nodes-1 % Segments, NOT nodes. 0-indexed
    
    for r=0:1:ratio-cut % RK sampling points through each segment
   
        index=shift*rk_step;
        
        in_l = s;   % index, left node
        in_r = s+1; % index, right node
   
        bank_l = bankangle(in_l); % bank angle, left node
        bank_r = bankangle(in_r); % bank angle, rigt node
        bank_d = bank_r-bank_l;   % delta bank angle
        % calculating bank angle input, linear interpolated 
        bank_i = bank_l + bank_d*r/ratio; % bank angle input
        
        acc_l = acceleration(in_l); % acceleration, left node
        acc_r = acceleration(in_r); % acceleration, rigt node
        acc_d = acc_r-acc_l;        % delta acceleration
        % calculating acceleration input, linear interpolated 
        acc_i = acc_l + acc_d*r/ratio; % acceleration input
        
        %k1=h*f(y_n) 
        RK_k1   = h*UAV_q_1_Functionvalue( y(1+index : svn+index),acc_i, bank_i ); %y_n
        %k2=h*f(y_n+k1/2) 
        y_k2 = y(1+index :shift+index)+RK_k1/2;
        RK_k2   = h*UAV_q_1_Functionvalue( y_k2(1:svn),acc_i, bank_i ); %
        %k3=h*f(y_n+k2/2) 
        y_k3 = y(1+index :shift+index) + RK_k2/2; 
        RK_k3   = h*UAV_q_1_Functionvalue( y_k3(1:svn), acc_i, bank_i );
        %k4=h*f(y_n+k3)
        y_k4 = y(1+index :shift+index) + RK_k3; 
        RK_k4   = h*UAV_q_1_Functionvalue(y_k4(1:svn), acc_i, bank_i );
   
        y( 1+shift*(rk_step+1):shift+shift*(rk_step+1) ) = y( 1+index : shift+index )+(RK_k1+2*RK_k2+2*RK_k3+RK_k4)/6;
        rk_step = rk_step+1;
    end
end

end
