%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%    Problem Setup for 1,2,4,9 uncertainty models                         %
%       Lars Grimsland, spring 2012 - lars.grimsland@gmail.com            %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% If changes to model and/or objective functions  are wanted - modifiy
% Uct_Constrintsetup, Uct_ObjectiveFcn, Uct_Functionvalue, and
% Uct_FunctionvalueSym

% Number of uncertainty models and segments are decided below
% Paths for where to put filse to be included in the C++ program are also set
clc;
clear all;

%% - Parameters - %%

% PROBLEM PARAMETERES TO HARDCODE %
segments = 30;
targets  = 2; % 1,2,4, or 9 
svn      = 4+targets;        % SVN : System variables pr node - UAV states + Number of target uncertainty models (UAV Inputs not included)
svn_str  = int2str(svn);
ovn      = svn+2;            % OVN : Optimization variables pr node - UAV inputs, UAV states and Uncertainty model states
ovn_str  = num2str(ovn);


% Block related constants are automaticly updated later on
jnb     = -1;           % JNB : Jacobian nonzeros pr block
jnb_str = num2str(jnb);
hnb     = -1;           % HNB : Hessian nonzeros first block
hnb_str = num2str(hnb);
hnba    = -1;           % HNBA: Hessian nonzeros added by block 2,3,...
hnba_str= num2str(hnba);

% Paths to textfile wich is written to by script, included in C++ progr.

 textfiles_path = 'C:\textfiles\';

g_ccode_path   = strcat(textfiles_path, 'g_ccode.txt');
JacVal_path    = strcat(textfiles_path, 'JacVal.txt');
JacPos_path    = strcat(textfiles_path, 'JacPos.txt');
objective_path = strcat(textfiles_path, 'objective.txt');
obj_jac_path   = strcat(textfiles_path, 'obj_jac.txt');
hesspos_path   = strcat(textfiles_path, 'hesspos.txt');
hessval2_path  = strcat(textfiles_path, 'hessval2.txt');
startval_path  = strcat(textfiles_path, 'startval.txt');

%% - Defining symbols for start point/defect vector calc - %%

syms x0 x1 x2 x3 x4 x5 x6 x7 x8 x9 x10 x11 x12 x13 x14 x15 
syms x16 x17 x18 x19 x20 x21 x22 x23 x24 x25 x26 x27 x28 x29
syms lambda0 lambda1 lambda2 lambda3  lambda4  lambda5  lambda6 
syms lambda7 lambda8 lambda9 lambda10 lambda11 lambda11 lambda12
syms symfix obj_factor 
syms T grav 

x_all = [x0;x1;x2;x3;x4;x5;x6;x7;x8;x9;x10;x11;x12;x13;x14;x15;x16;x17;x18;x19;x20;x21;x22;x23;x24;x25;x26;x27;x28;x29];
x=x_all(1:2*(ovn));
syms alpha0 b0 k0 T0n T0e ceta0
syms alpha1 b1 k1 T1n T1e ceta1
syms alpha2 b2 k2 T2n T2e ceta2
syms alpha3 b3 k3 T3n T3e ceta3
syms alpha4 b4 k4 T4n T4e ceta4
syms alpha5 b5 k5 T5n T5e ceta5
syms alpha6 b6 k6 T6n T6e ceta6
syms alpha7 b7 k7 T7n T7e ceta7
syms alpha8 b8 k8 T8n T8e ceta8

y0      = [x0;x1;x2;x3;x4;x5;x6;x7;x8;x9;x10;x11;x12;];
alpha   = sym([alpha0;alpha1;alpha2;alpha3;alpha4;alpha5;alpha6;alpha7;alpha8;]);
b       = sym( [b0;b1;b2;b3;b4;b5;b6;b7;b8;]  );
ceta    = sym( [ceta0;ceta1;ceta2;ceta3;ceta4;ceta5;ceta6;ceta7;ceta8;] );
k       = sym( [k0;k1;k2;k3;k4;k5;k6;k7;k8;] ); 
Te      = sym( [T0e;T1e;T2e;T3e;T4e;T5e;T6e;T7e;T8e;] );
Tn      = sym( [T0n;T1n;T2n;T3n;T4n;T5n;T6n;T7n;T8n;] );
 


%% - Optimization start point - %%
Uct_StartPoint;

%% - Uav model, COLLOCATION defect vector / constraints calculated - %% 
Uct_ConstraintSetup;
                    
%% - Writing constraints to file (as C code)-  %%
Uct_ConstraintWrite;

%% - Constraints jacobian- %% 
Uct_ConstraintJacobian;

%% - Objective Functions - %%
Uct_ObjectiveFcn;

%% - Gradient of objective function - %% 
Uct_ObjectiveFcnGradient;

%% Hessian of Lagrange Function %%
Uct_Hessian