%% - Uav model, defect vector calculation%% 
%
% x0 = x(1)  : North pos. coordinate [m]
% x1 = x(2)  : East pos. coordinate [m]
% x2 = x(3)  : Uav speed [m/s]
% x3 = x(4)  : Uav heading [radian]
% x4 = x(5)  : Uncertainty - target 1
% x5 = x(6)  : Uncertainty - target 2
%  .......
% x.. : Uav Input 1 - Acceleration [m/s^2]
% x...: Uav Input 2 - Bank angle [radians]

disp('Calculating defect vector')

% Preallocating vector%
fx_l = sym(zeros(4+targets,1));
fx_r = sym(zeros(4+targets,1));
fx_c = sym(zeros(4+targets,1));

% The state and input vectors %
if (targets == 1)
    disp('A singe target / uncertainty model chosen')
    x_l=[ x0; x1; x2; x3; x4;];  % state variables, left node
    u_l=[ x5; x6;];                  % inputs, left node
    x_r=[ x7; x8; x9;x10;x11;];  % state variables, right node
    u_r=[x12;x13;];
elseif (targets == 2)
    disp('Two targets / uncertainty models chosen')
    % Model - TWO TARGETS %
    x_l=[ x0; x1; x2; x3; x4; x5;];  % state variables, left node
    u_l=[ x6; x7;];                  % inputs, left node
    x_r=[ x8; x9;x10;x11;x12;x13;];  % state variables, right node
    u_r=[x14;x15;];                  % inputs, right node
  
elseif (targets==4)
    disp('Four targest / uncertainty models chosen')
        % Model - FOUR TARGETS %
    x_l=[ x0; x1; x2; x3; x4; x5; x6; x7];  % state variables, left node
    u_l=[ x8; x9;];                  % inputs, left node
    x_r=[ x10;x11;x12;x13;x14;x15;x16;x17;];  % state variables, right node
    u_r=[ x18;x19;];                  % inputs, right node   
elseif (targets==9)
    disp('Nine targest / uncertainty models chosen')
    x_l=[ x0; x1; x2; x3; x4; x5; x6; x7;x8;x9;x10;x11;x12;];  % state variables, left node
    u_l=[ x13; x14;];                  % inputs, left node
    x_r=[ x15;x16;x17;x18;x19;x20;x21;x22;x23;x24;x25;x26;x27;];  % state variables, right node
    u_r=[ x28;x29;];    
end


% MODEL: system eq., left node                     
    fx_l(1) =    x_l(3)*cos(x_l(4));         % x0_dot=..., x0 = north 
    fx_l(2) =    x_l(3)*sin(x_l(4));         % x1_dot=..., x1 = east
    fx_l(3) =    u_l(1);                   % x2_dot=..., x2 = speed
    fx_l(4) =    grav*tan(u_l(2))/x_l(3);    % x3_dot=..., x3 = heading
    for i=1:1:targets           
        fx_l(4+i) = x_l(4+i)*(alpha(i)-b(i)*exp(-k(i)*sqrt((Tn(i)-x_l(1))^2+(Te(i)-x_l(2))^2))) + ceta(i); 
    end           
% MODEL: system eq., left node                     
    fx_r(1) =    x_r(3)*cos(x_r(4));         % x0_dot=..., x0 = north 
    fx_r(2) =    x_r(3)*sin(x_r(4));         % x1_dot=..., x1 = east
    fx_r(3) =    u_r(1);                     % x2_dot=..., x2 = speed
    fx_r(4) =    grav*tan(u_r(2))/x_r(3);    % x3_dot=..., x3 = heading
    for i=1:1:targets           
        fx_r(4+i) = x_r(4+i)*(alpha(i)-b(i)*exp(-k(i)*sqrt((Tn(i)-x_r(1))^2+(Te(i)-x_r(2))^2))) + ceta(i); 
    end 

    x_c = (x_l+x_r)/2 + T*(fx_l-fx_r)/8;    % state vector at collocation point
    u_c = (u_l + u_r )/2;                   % input vector at collocation point
    x_c_dot = (-3)*(x_l-x_r)/(2*T) - (fx_l+fx_r)/4 ;

% MODEL: system eq., CENTER                     
    fx_c(1) =    x_c(3)*cos(x_c(4));         % x0_dot=..., x0 = north 
    fx_c(2) =    x_c(3)*sin(x_c(4));         % x1_dot=..., x1 = east
    fx_c(3) =    u_c(1);                     % x2_dot=..., x2 = speed
    fx_c(4) =    grav*tan(u_c(2))/x_c(3);    % x3_dot=..., x3 = heading
    for i=1:1:targets           
        fx_c(4+i) = x_c(4+i)*(alpha(i)-b(i)*exp(-k(i)*sqrt((Tn(i)-x_c(1))^2+(Te(i)-x_c(2))^2))) + ceta(i); 
    end
   

g = fx_c - x_c_dot; % calculating defect vector. 
                    % defects = constraints, called g[] in IPOPT
