%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Uct2_RK4_SV.m - y=Uct2_RK4_SV(y0, u0, u1, T)
% 
% An RK4 method, returning the value of next node, given initial values
% and inputs. Suited for UAV + 2 uncertainties/targets. Symbolic calcuation
%
% Lars Grimsland, spring 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function y = Uct_RK4_SV(y0, u0, u1, T,alpha,b,ceta,k,Te,Tn,targets)
% y0 : symbol array, initial state values (x0,x1,..,x5) 
% u0 : first input (acc)
% u1 : second input (bank)
    
%k1=h*f(y_n) 
f_k1 =sym( T*Uct_FunctionvalueSym( y0(1:4+targets),u0, u1,alpha,b,ceta,k,Te,Tn,targets )); %y_n

%k2=h*f(y_n+k1/2) 
y_k2 = sym(y0(1:4+targets)+f_k1/2);
f_k2   = sym(T*Uct_FunctionvalueSym(y_k2, u0, u1,alpha,b,ceta,k,Te,Tn,targets )); %

%k3=h*f(y_n+k2/2) 
y_k3 = sym(y0(1:4+targets) + f_k2/2); 
f_k3 = sym (T*Uct_FunctionvalueSym(y_k3, u0, u1,alpha,b,ceta,k,Te,Tn,targets ));

%k4=h*f(y_n+k3)
y_k4 = sym( y0(1:4+targets) + f_k3); 
f_k4 = sym( T*Uct_FunctionvalueSym(y_k4, u0, u1,alpha,b,ceta,k,Te,Tn,targets ));

% RK4: y(n+1)= y(n)+(k1+2*k2+2*k3+k4)/6;
y = sym(y0(1:4+targets) +(f_k1+2*f_k2+2*f_k3+f_k4)/6);


end
