%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                        %
%   Uct2NMPC_PlotSim.m                                                   %
%   Reads results from IPOPT program from text file,                     %
%   MENT FOR NMPC DEMO WITH 2 MOVING TARGETS                             %
%                                                                        %
%   Lars Grimsland, spring 2011, lars.grimsland@gmail.com                %
%                                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc;

%% Reading various PARMATETERS from optimization %%
%parameter_id = fopen('C:\Users\lars.grimsland\Dropbox\ShortFlight - 2ExpBell\Debug\parameters.txt', 'r');
parameter_id = fopen('C:\Users\grimslan\Dropbox\ShortFlight - 2ExpBell NMPC\Release\parameters.txt', 'r');
% Problem data %
tline = fgetl(parameter_id);
model_states = str2double(tline);
tline        = fgetl(parameter_id);
model_inputs = str2double(tline);
tline        = fgetl(parameter_id);
T            = str2double(tline);
tline        = fgetl(parameter_id);
RecalcNode   = str2double(tline);

fclose(parameter_id);


%% Reading Result of optimization on first NMPC iteration from file, sorting them %%
org_north_c = [];    % Array of UAV North position coordinates
org_east_c  = [];    % Array of UAV East position coordinates

orgpath_id = fopen('C:\Users\grimslan\Dropbox\ShortFlight - 2ExpBell NMPC\Release\originalpath.txt', 'r');
counter=0;
while 1
    tline = fgetl(orgpath_id);
    if ~ischar(tline), break, end
        x = str2double(tline);
        counter = counter+1;
        index = mod(counter,8);
        if (index==1)
            org_north_c = [org_north_c;x;];
        elseif (index==2)
            org_east_c = [org_east_c;x;];
        end
            
end    
fclose(orgpath_id);

%%  - Reading ice/targets positions - %%
T0_north_c = [];    % Array of Target 0 North position coordinates
T0_east_c  = [];    % Array of Target 1 East position coordinates

T1_north_c = [];    % Array of Target 0 North position coordinates
T1_east_c  = [];    % Array of Target 1 East position coordinates

ice_id = fopen('C:\Users\grimslan\Dropbox\ShortFlight - 2ExpBell NMPC\Release\icepos.txt', 'r');
counter=0;
while 1
    tline = fgetl(ice_id);
    if ~ischar(tline), break, end
        x = str2double(tline);
        counter = counter+1;
        index = mod(counter,4);
        if (index==1)
            T0_north_c = [T0_north_c;x;];
        elseif (index==2)
            T0_east_c = [T0_east_c;x;];
        elseif (index==3)
            T1_north_c = [T1_north_c;x;];
        elseif (index==0)
            T1_east_c = [T1_east_c;x;];
        end
            
end    
fclose(ice_id);


%% Reading optimization Result from file, sorting them %%

% _Not_ preallocating to make it as simple as possible
north_c      = [];       % Array of UAV North position coordinates
east_c       = [];          % Array of UAV East position coordinates
uav_speed    = [];       % Array of UAV air speeds
uav_heading  = [];     % Array of UAV headings
tar_uncert0  = [];      % Array of Ice field survailance information uncertainty
tar_uncert1  = [];
acceleration = [];    % Array of UAV input acceleration
bank_angle   = [];      % Array of UAV input banking angle
counter      = 0;

%example_id = fopen('C:\Users\lars.grimsland\Dropbox\ShortFlight - 2ExpBell NMPC\Debug\example2.txt', 'r');
example_id = fopen('C:\Users\grimslan\Dropbox\ShortFlight - 2ExpBell NMPC\Release\opt_solution.txt', 'r');
while 1
    tline = fgetl(example_id)
    if ~ischar(tline), break, end
        x = str2double(tline)
        counter = counter+1;
        index = mod(counter,8)
        if (index==1)
            north_c = [north_c;x;];
        elseif (index==2)
            east_c = [east_c;x;];
        elseif (index==3)
            uav_speed = [uav_speed; x;];
        elseif (index==4)
            uav_heading = [uav_heading; x;];
        elseif (index==5)
            tar_uncert0 = [tar_uncert0;x;];
        elseif (index==6)
            tar_uncert1 = [tar_uncert1;x;];
         elseif (index==7)
            acceleration = [acceleration;x;];
        elseif (index==0)
            bank_angle = [bank_angle;x;];
        else
                disp('unecspected occurance while sorting results')
        end
            
end    
fclose(example_id);

%% Plotting the sorted results %%
temp=size(north_c);
N=temp(1);  % N: number of nodes
flytime=(N-1)*T;
timescaler=0:T:(N-1)*T ;


%Plot the optimal path
figure(2)
    plot(east_c,north_c, 'b-o', 'LineWidth', 2)
    title(['Optimal Path w/' num2str(N-1),' nodes and ', num2str(flytime), ' sec flight time.']) ;
    xlabel('Metres East, [m]')
    ylabel('Metres North, [m]')
    grid on
    hold on
    plot(org_east_c,org_north_c, 'r-o', 'LineWidth', 2)
    plot(T0_east_c,T0_north_c, 'm-o', 'LineWidth', 2)
    plot(T1_east_c,T1_north_c, 'm-o', 'LineWidth', 2)
    legend('Flewn UAV path','First calculated path','Targets', 'Location', 'SouthEast')
    hold off
    
% Plot other optimized variables    
figure( 3)    
    subplot(2,1,1),plot(timescaler, uav_heading, 'b-o','LineWidth', 2),title('Heading; (0 degrees = North, 90 degrees = East- )')
        xlabel('Time, [s]')
        ylabel('Heading, [degrees]')
        grid on
    subplot(2,1,2),plot(timescaler,uav_speed, 'b-o', 'LineWidth', 2),title('Groundspeed')
        xlabel('Time, [s]')
        ylabel('Speed, [m/s]')
        grid on
figure(4)    
    subplot(2,1,1)
        stairs(timescaler,acceleration, 'b-o','LineWidth', 2),title('u1 - commanded acceleration')
        xlabel('Time, [s]')
        ylabel('Acceleration, [m/s^2]')
        grid on
    subplot(2,1,2)
        stairs(timescaler,bank_angle, 'b-o','LineWidth', 2), title('u2 - commanded bank angle')
        xlabel('Time, [s]')
        ylabel('Bank angle, [degrees]')
        grid on
figure(5) % Plotting Graphs of uncertainties in the same figure
        stairs(timescaler,tar_uncert0, 'b-o','LineWidth', 2), title('Information Uncertainty of Targets')
        xlabel('Time, [s]')
        ylabel('Uncertainty')
        grid on
        hold on
        stairs(timescaler,tar_uncert1, 'r-o','LineWidth', 2)
        legend('x5 - Uncertainty, Target 0','x6 - Uncertainty, Target 1', 'Location', 'NorthEast')
        grid on
        hold off

%% Computation of crap %%
T0e_pos_all=[];
T0n_pos_all=[];
T1e_pos_all=[];
T1n_pos_all=[];
temp = size(T0_east_c);
legs=temp(1);
for i=1:1:legs
    T0e_pos_all( (i-1)*RecalcNode+1  : i*RecalcNode ) = ones(RecalcNode,1)*T0_east_c(i) ;
    T0n_pos_all( (i-1)*RecalcNode+1  : i*RecalcNode ) = ones(RecalcNode,1)*T0_north_c(i) ;
    T1e_pos_all( (i-1)*RecalcNode+1  : i*RecalcNode ) = ones(RecalcNode,1)*T1_east_c(i) ;
    T1n_pos_all( (i-1)*RecalcNode+1  : i*RecalcNode ) = ones(RecalcNode,1)*T1_north_c(i) ;
end

T0e_pos_all=reshape(T0e_pos_all,[],1);
T0n_pos_all=reshape(T0n_pos_all,[],1);
T1e_pos_all=reshape(T1e_pos_all,[],1);
T1n_pos_all=reshape(T1n_pos_all,[],1);


distT0 = sqrt((T0e_pos_all-east_c).^2 + (T0n_pos_all-north_c).^2);
distT1 = sqrt((T1e_pos_all-east_c).^2 + (T1n_pos_all-north_c).^2);


figure(6) % Plotting Graphs of uncertainties in the same figure
        stairs(timescaler,distT0, 'b-o','LineWidth', 2), title('Distance from UAV to targets')
        xlabel('Time, [s]')
        ylabel('meters [m]')
        grid on
        hold on
        stairs(timescaler,distT1, 'r-o','LineWidth', 2)
        legend('Distance to Target 0','Distance to Target 1', 'Location', 'NorthEast')
        grid on
        hold off


disp(' Components in Objective Function ')
sum_uncert_target_0 = sum( tar_uncert0 );
sum_uncert_target_1 = sum( tar_uncert1 );
sum_acc_effort = sum(acceleration);
sum_bank_effort = sum(bank_angle);

disp('Integral of Target 0 uncertainty: ');
disp( sum_uncert_target_0)%,'    Wheight: ', Wu0)
disp('Integral of Target 1 uncertainty: ');
disp(sum_uncert_target_1)%,'    Wheight: ', Wu1)
disp('Integral of Acceleration:         ');
disp( sum_acc_effort)%,'    Wheight: ', Wa)
disp('Integral of Bank Angle  :         ' );
disp(sum_bank_effort)%,'    Wheight: ', Wb)
        
