/*
	MAIN proram file NMPC with target update
*/

#include "IpIpoptApplication.hpp"
#include "short_nlp.hpp"
#include "parameters.h"

#define PAUSE {printf("Press Enter to continue...\n"); fflush(stdin); getchar(); fflush(stdin);}

#include <iostream>

using namespace Ipopt;


int main(int argv, char* argc[])
{

  // Create a new instance of your nlp
  //  (use a SmartPtr, not raw)
  SmartPtr<TNLP> mynlp = new short_NLP();
  std::cout << "DERP ..... step: 1 \n";
  
  // Create a new instance of IpoptApplication
  //  (use a SmartPtr, not raw)
  // We are using the factory, since this allows us to compile this
  // example with an Ipopt Windows DLL
  SmartPtr<IpoptApplication> app = IpoptApplicationFactory();
  std::cout << "DERP ..... step: 2 \n";
  
  // Change some options
  // Note: The following choices are only examples, they might not be
  //       suitable for your optimization problem.
  app->Options()->SetStringValue("mu_strategy", "adaptive");
  app->Options()->SetStringValue("output_file", "ipopt.out");
  
  /* Exact Hessian vs BFGS */
  app->Options()->SetStringValue("hessian_approximation", "exact");
  //app->Options()->SetStringValue("hessian_approximation", "limited-memory");
  //app->Options()->SetStringValue("limited_memory_update_type", "bfgs");

  /* Derivative Checker */
  //app->Options()->SetStringValue("check_derivatives_for_naninf", "yes");
  //app->Options()->SetStringValue("derivative_test", "first-order"); //second-order, first-order

  /* Evaluations/step lengths */
  //app->Options()->SetIntegerValue("max_iter", 4000); // perturbation in derivative test, Default 1.0E-8

  
  // Intialize the IpoptApplication and process the options
  ApplicationReturnStatus status;
  status = app->Initialize();
   if (status != Solve_Succeeded) {
    std::cout << std::endl << std::endl << "*** Error during initialization!" << std::endl;
    return (int) status;
  }


  for (int mpc_loop = 0; mpc_loop < get_noMpcIter(); mpc_loop++){ //start of MPC-LOOP
		std::cout << "MPC-LOOP: " << mpc_loop  << " \n";
		
		// Ask Ipopt to solve the problem
		status = app->OptimizeTNLP(mynlp);
  
		std::cout << "DERP ..... step: 6 \n";
		if (status == Solve_Succeeded) {
			std::cout << std::endl << std::endl << "*** The problem solved!" << std::endl;
		}
		else {
			std::cout << std::endl << std::endl << "*** The problem FAILED!" << std::endl;
		}

		// As the SmartPtrs go out of scope, the reference count
		// will be decremented and the objects will automatically
		// be deleted.


		set_incrMpcLoopNr();	
	} // End of NMPC-LOOP
	
	PAUSE

  return (int) status;
}
