/*
    Object holding NLP information
*/

#include "short_NLP.hpp"
#include "parameters.h"

#include <cassert>
#include <iostream>
#include <fstream>

//#include <math.h>
#include <cmath>

/* Redifining variables used in included files */
/* x_n = x[n+1*(optimization vars pr node)] */
#define x10 x[10+i*7]
#define x11 x[11+i*7]
#define x12 x[12+i*7]
#define x13 x[13+i*7]
#define x14 x[14+i*7]
#define x15 x[15+i*7]

#define x0  x[0+i*7] // 0,6,12,18....
#define x1  x[1+i*7]
#define x2  x[2+i*7]
#define x3  x[3+i*7]
#define x4  x[4+i*7]
#define x5  x[5+i*7]
#define x6  x[6+i*7]
#define x7  x[7+i*7]
#define x8  x[8+i*7]
#define x9  x[9+i*7]

/* Lagr. mulitpliers: [n+i*(constraints pr segment)]*/
#define lambda0 lambda[0+i*5]
#define lambda1 lambda[1+i*5]
#define lambda2 lambda[2+i*5]
#define lambda3 lambda[3+i*5]
#define lambda4 lambda[4+i*5]
#define lambda5 lambda[5+i*5]

#define symfix 1


using namespace Ipopt;

// constructor
short_NLP::short_NLP()
{}

//destructor
short_NLP::~short_NLP()
{}

// returns the size of the problem
bool short_NLP::get_nlp_info(Index& n, Index& m, Index& nnz_jac_g,
                             Index& nnz_h_lag, IndexStyleEnum& index_style)
{
	n = get_noVar();	// parameters defined in parameters.cpp
	m = get_noConstr();
	
	nnz_jac_g = get_conJacNonZ(); // number of nonzeros in jacobian of constraints
	nnz_h_lag = get_lagrHessNonZ(); // nonzeros in lower left corner og lagrang hessian
	
	// use the C style indexing (0-based)
	index_style = TNLP::C_STYLE;
	
	return true;
}

// returns the variable bounds
bool short_NLP::get_bounds_info(Index n, Number* x_l, Number* x_u,
                                Index m, Number* g_l, Number* g_u)
{
	//std::cout << " variable bounds begynt\n";
	// here, the n and m we gave IPOPT in get_nlp_info are passed back to us.
	// If desired, we could assert to make sure they are what we think they are.
	int i;
	int limit=get_segments();
	assert(n == get_noVar());
	assert(m == get_noConstr());

	// the variables have lower bounds of 1
	double lower_var_bounds[7] = {-2e19, -2e19, 11.2, -2e19,  -2e19, -3.00, -0.523598};
	//                            North  East   Speed  Dir      Unc    Acc     Bank

	// lower (and upper) constraints on the first node is used to fix a startpoint for the flight path 
    x_l[0] = get_startPosUavNorth();		// lower_var_bounds[0];
	x_l[1] = get_startPosUavEast();		// lower_var_bounds[1];
	x_l[2] = get_startUavSpeed();	// lower_var_bounds[2];
	x_l[3] = get_startUavHeading();		//lower_var_bounds[3];
	x_l[4] = get_startValUncertainty0();		//lower_var_bounds[4];
	x_l[5] = lower_var_bounds[5];
	x_l[6] = lower_var_bounds[6];


	
	for(i=0; i<limit; i++){
		x_l[ 7+i*7]  = lower_var_bounds[0]; //x_l[ 6, 12, 18...] 	
		x_l[ 8+i*7]  = lower_var_bounds[1];
		x_l[9+i*7] = lower_var_bounds[2];
		x_l[10+i*7] = lower_var_bounds[3];
		x_l[11+i*7] = lower_var_bounds[4];
		x_l[12+i*7] = lower_var_bounds[5];
		x_l[13+i*7] = lower_var_bounds[6];
		
	}

  // the variables have upper bounds
	double upper_var_bounds[7] = {2e19, 2e19, 26.8, 2e19,  2e19, 3.00, 0.523598};
    //                           North  East  Speed Dir     Unc    Acc    Bank

    x_u[0] = get_startPosUavNorth();		
	x_u[1] = get_startPosUavEast();	
	x_u[2] = get_startUavSpeed();		
	x_u[3] = get_startUavHeading();			
	x_u[4] = get_startValUncertainty0();	
	x_u[5] = upper_var_bounds[5];
	x_u[6] = upper_var_bounds[6];
	
	i=0;
	for(i=0; i< limit; i++){
		x_u[ 7+i*7] = upper_var_bounds[0]; //x_u[ 6, 12, 18...]
		x_u[ 8+i*7] = upper_var_bounds[1];
		x_u[ 9+i*7] = upper_var_bounds[2];
		x_u[10+i*7] = upper_var_bounds[3];
		x_u[11+i*7] = upper_var_bounds[4];
		x_u[12+i*7] = upper_var_bounds[5];
		x_u[13+i*7] = upper_var_bounds[6];
		
	}
  

   // the second constraint g2 is an equality constraint, so we set the
  // upper and lower bound to the same value
 
  for(int i=0; i<(m) ;i++){
	g_l[i] = 0; // all constraints are equality constraints
	g_u[i] = 0;
  }

  return true;
}

// returns the initial point for the problem
bool short_NLP::get_starting_point(Index n, bool init_x, Number* x,
                                   bool init_z, Number* z_L, Number* z_U,
                                   Index m, bool init_lambda,
                                   Number* lambda)
{
  // Here, we assume we only have starting values for x, if you code
  // your own NLP, you can provide starting values for the dual variables
  // if you wish
	
	int limit=get_segments();
	double  T = get_T();
	
	double	alpha0 = get_alpha0();
	double	b0 = get_b0();
	double	k0 = get_k0();
	double ceta0 = get_ceta0();
	double	T0n= get_T0N();	// returns north coordinate of grid center 0
	double	T0e= get_T0E();	// returns east coordinate of grid center 0
	
	double	alpha1 = get_alpha1();
	double	b1 = get_b1();
	double	k1 = get_k1();
	double ceta1 = get_ceta1();
	double	T1n= get_T1N();	// returns north coordinate of grid center 1
	double	T1e= get_T1E();	// returns east coordinate of grid center 1
	

	assert(init_x == true);
	assert(init_z == false);
	assert(init_lambda == false);

// values in the first node
	x[0] = get_startPosUavNorth();
	x[1] = get_startPosUavEast();
	x[2] = get_startUavSpeed();
	x[3] = get_startUavHeading();
	x[4] = get_startValUncertainty0();

	x[5] = 0; //inputs are zero to ease start point calculation
	x[6] = 0; 

  // initialize to the given starting point, 
	for(int i=0; i<get_segments(); i++){
		#include "C:\textfiles\startval.txt"
		//#include "C:\Users\lars.grimsland\Dropbox\ShortFlight - 2ExpBell\textfiles\startval.txt"
	}
	
	return true;
}

// returns the value of the objective function
bool short_NLP::eval_f(Index n, const Number* x, bool new_x, Number& obj_value)
{
	assert(n == get_noVar());
	double Wa = get_Wa(); // returns wheigting (punishments) on input 1, acceleration
	double Wr = get_Wr(); // returns wheigting (punishments) on input 2, rudder/bank angle
	double Wu0 = get_Wu0(); // returns wheigting (punishments) on UNCERTAINTY
	double Wu1 = get_Wu1();
	double Wu2 = get_Wu2();
	double Wu3 = get_Wu3();
	double Wu4 = get_Wu4();
	double Wu5 = get_Wu5();
	double Wu6 = get_Wu6();
	double Wu7 = get_Wu7();
	double Wu8 = get_Wu8();

	#include "C:\textfiles\objective.txt"
	//#include "C:\Users\lars.grimsland\Dropbox\ShortFlight - 2ExpBell\textfiles\objective.txt"
	
	return true;
}

// return the gradient of the objective function grad_{x} f(x)
bool short_NLP::eval_grad_f(Index n, const Number* x, bool new_x, Number* grad_f)
{
	assert(n == get_noVar()); //nodes*system_variables);
	double Wa = get_Wa(); // returns wheigting (punishments) on input 1, acceleration
	double Wr = get_Wr(); // returns wheigting (punishments) on input 2, rudder/bank angle
	double Wu0 = get_Wu0(); // returns wheigting (punishments) on UNCERTAINTY
	double Wu1 = get_Wu1();
	double Wu2 = get_Wu2();
	double Wu3 = get_Wu3();
	double Wu4 = get_Wu4();
	double Wu5 = get_Wu5();
	double Wu6 = get_Wu6();
	double Wu7 = get_Wu7();
	double Wu8 = get_Wu8();
	
	#include "C:\textfiles\obj_jac.txt"
	//#include "C:\Users\lars.grimsland\Dropbox\ShortFlight - 2ExpBell\textfiles\obj_jac.txt"
		
	return true;
}

// return the value of the constraints: g(x)
bool short_NLP::eval_g(Index n, const Number* x, bool new_x, Index m, Number* g)
{
	//std::cout << " constraints value begin  \n" ;
	assert(n == get_noVar());
	assert(m == get_noConstr());
	double T = get_T();
	double grav = get_grav();

	double alpha0 =	get_alpha0();
	double b0	= get_b0();
	double k0	= get_k0();
	double ceta0	 = get_ceta0();
	double T0n	= get_T0N();
	double T0e	= get_T0E();
	
	double alpha1 =	get_alpha0();
	double b1	= get_b0();
	double k1	= get_k0();
	double ceta1 = get_ceta1();
	double T1n	= get_T1N();
	double T1e	= get_T1E();

	double alpha2 =	get_alpha2();
	double b2	= get_b2();
	double k2	= get_k2();
	double ceta2 = get_ceta2();
	double T2n	= get_T2N();
	double T2e	= get_T2E();

	double alpha3 =	get_alpha3();
	double b3	= get_b3();
	double k3	= get_k3();
	double ceta3 = get_ceta3();
	double T3n	= get_T3N();
	double T3e	= get_T3E();
		
	int i=0;
	int limit = get_segments();
	for(i=0; i<limit; i++){
			#include "C:\textfiles\g_ccode.txt"
			//#include "C:\Users\lars.grimsland\Dropbox\ShortFlight - 2ExpBell\textfiles\g_ccode.txt"
	}
	return true;
}

// return the structure or values of the constraints jacobian
bool short_NLP::eval_jac_g(Index n, const Number* x, bool new_x,
                           Index m, Index nele_jac, Index* iRow, Index *jCol,
                           Number* values)
{
	int i=0;
	int limit = get_segments();
	
	if (values == NULL) {   // return the structure of the jacobian
		
		for(i=0; i<limit; i++){
				#include "C:\textfiles\JacPos.txt"
				//#include "C:\Users\lars.grimsland\Dropbox\ShortFlight - 2ExpBell\textfiles\JacPos.txt"
		}
  }
  else {
    // return the values of the jacobian of the constraints
	double T	 = get_T();
	double grav  = get_grav();

	double alpha0 = get_alpha0();
	double b0	 = get_b0();
	double k0	 = get_k0();
	double ceta0 = get_ceta0();
	double T0n	 = get_T0N();
	double T0e	 = get_T0E();

	double alpha1 = get_alpha1();
	double b1	 = get_b1();
	double k1	 = get_k1();
	double ceta1 = get_ceta1();
	double T1n	 = get_T1N();
	double T1e	 = get_T1E();

	double alpha2 = get_alpha2();
	double b2	 = get_b2();
	double k2	 = get_k2();
	double ceta2 = get_ceta2();
	double T2n	 = get_T2N();
	double T2e	 = get_T2E();

	double alpha3 = get_alpha3();
	double b3	 = get_b3();
	double k3	 = get_k3();
	double ceta3 = get_ceta3();
	double T3n	 = get_T3N();
	double T3e	 = get_T3E();
	
    
	for(i=0; i<limit; i++){
			#include "C:\textfiles\JacVal.txt"
			//#include "C:\Users\lars.grimsland\Dropbox\ShortFlight - 2ExpBell\textfiles\JacVal.txt"
	}
  }
	//std::cout << "done with jacobian \n";
	return true;
}

//return the structure or values of the hessian
bool short_NLP::eval_h(Index n, const Number* x, bool new_x,
                       Number obj_factor, Index m, const Number* lambda,
                       bool new_lambda, Index nele_hess, Index* iRow,
                       Index* jCol, Number* values)
{
	//std::cout << "Hessian begun \n";
	double T	= get_T();			// segment length
	double grav	= get_grav();
	/*Obj. fcn. Wheigts*/
	double Wa = get_Wa(); // returns wheigting (punishments) on input 1, acceleration
	double Wr = get_Wr(); // returns wheigting (punishments) on input 2, rudder/bank angle
	double Wt = get_Wt(); // returns wheigting (punishments) on Target distance
	double Wu0 = get_Wu0();
	double Wu1 = get_Wu1();
	double Wu2 = get_Wu2();
	double Wu3 = get_Wu3();
	double Wu4 = get_Wu4();
	double Wu5 = get_Wu5();
	double Wu6 = get_Wu6();
	double Wu7 = get_Wu7();
	double Wu8 = get_Wu8();
	/*Target1*/
	double alpha0 = get_alpha0();
	double b0	  = get_b0();
	double k0	  = get_k0();// various parameter in
	double ceta0  = get_ceta0(); // uncertainty model
	double T0n	  = get_T0N();		
	double T0e	  = get_T0E();

	double alpha1 = get_alpha1();
	double b1	  = get_b1();
	double k1	  = get_k1();// various parameter in
	double ceta1  = get_ceta1(); // uncertainty model
	double T1n	  = get_T1N();		
	double T1e	  = get_T1E();

	double alpha2 = get_alpha2();
	double b2	  = get_b2();
	double k2	  = get_k2();// various parameter in
	double ceta2  = get_ceta2(); // uncertainty model
	double T2n	  = get_T2N();		
	double T2e	  = get_T2E();

	double alpha3 = get_alpha3();
	double b3	  = get_b3();
	double k3	  = get_k3();// various parameter in
	double ceta3  = get_ceta3(); // uncertainty model
	double T3n	  = get_T3N();		
	double T3e	  = get_T3E();
		/*Prep for Hessian block generalization*/
	double valuebase[140];
	int v_index;

	if (values == NULL) {
		// return the structure. This is a symmetric matrix, fill the lower left
		// triangle only.
	
	    #include "C:\textfiles\hesspos.txt"
		//#include "C:\Users\lars.grimsland\Dropbox\ShortFlight - 2ExpBell\textfiles\hesspos.txt"
		
	}
	else {
		// return the values. This is a symmetric matrix, fill the lower left triangle only
		/*Including a single Hessian Block , generalizing values*/
	
		for(int i=0; i<get_segments(); i++ ){		// i couts segments
			#include "C:\textfiles\hessval2.txt"
			for(int nn=0; nn<get_lagrHessNonzBlock(); nn++){
				if(i==0){
					/*No appending in first block*/
					values[nn]=valuebase[nn];
				}	
				else if (i>0 && nn<get_HessNoOverlaps() ){
					v_index=(i-1)*(get_lagrHessNonzBlock()-get_HessNoOverlaps())+get_HessOverlapIndex(nn);
					values[v_index]=values[v_index]+valuebase[nn];
				}
				else{
				v_index = nn+(get_lagrHessNonzBlock()-get_HessNoOverlaps())*i;
				values[v_index]=valuebase[nn];
				}
			} // for loop counting Hess Nonzeros nn ends
		} // for loop counting segments, i ends  
	} // else-statement ends
  return true;
}


void short_NLP::finalize_solution(SolverReturn status,
                                  Index n, const Number* x, const Number* z_L, const Number* z_U,
                                  Index m, const Number* g, const Number* lambda,
                                  Number obj_value,
								  const IpoptData* ip_data,
								  IpoptCalculatedQuantities* ip_cq)
{
  // here is where we would store the solution to variables, or write to a file, etc
  // so we could use the solution.

/*
  // write the solution to the console
  std::cout << std::endl << std::endl << "Solution of the primal variables, x" << std::endl;
  for (Index i=0; i<n; i++) {
     std::cout << "x[" << i << "] = " << x[i] << std::endl;
  }
*/

 //  Writing solution to file 
    std::ofstream myfile;
	myfile.open ("optimal_solution.txt");
	for (Index i=0; i<n; i++) {
		myfile << x[i] << std::endl;
	}
	myfile.close();

	std::ofstream parameter_file;
	parameter_file.open ("parameters.txt");

	/* Problem data */
	parameter_file << get_StateVarsNode() << std::endl;
	parameter_file << get_InputVarsNode() << std::endl;
	parameter_file << get_T()			  << std::endl;
	parameter_file << get_recalcNodeNo()  << std::endl;
	
	/* Uncertainty model and target 0 parameteres*/
	parameter_file << get_alpha0()	<< std::endl;
	parameter_file << get_b0()		<< std::endl;
	parameter_file << get_k0()		<< std::endl;
	parameter_file << get_ceta0()	<< std::endl;
	parameter_file << get_T0N()		<< std::endl;
	parameter_file << get_T0E()		<< std::endl;
	
	/* Uncertainty model and target 1 parameteres*/
	parameter_file << get_alpha1()	<< std::endl;
	parameter_file << get_b1()		<< std::endl;
	parameter_file << get_k1()		<< std::endl;
	parameter_file << get_ceta1()	<< std::endl;
	parameter_file << get_T1N()		<< std::endl;
	parameter_file << get_T1E()		<< std::endl;

	/* Uncertainty model and target 2 parameteres*/
	parameter_file << get_alpha2()	<< std::endl;
	parameter_file << get_b2()		<< std::endl;
	parameter_file << get_k2()		<< std::endl;
	parameter_file << get_ceta2()	<< std::endl;
	parameter_file << get_T2N()		<< std::endl;
	parameter_file << get_T2E()		<< std::endl;

	/* Uncertainty model and target 3 parameteres*/
	parameter_file << get_alpha3()	<< std::endl;
	parameter_file << get_b3()		<< std::endl;
	parameter_file << get_k3()		<< std::endl;
	parameter_file << get_ceta3()	<< std::endl;
	parameter_file << get_T3N()		<< std::endl;
	parameter_file << get_T3E()		<< std::endl;
	
	/* Objective function parametres*/
	parameter_file << get_Wa()  << std::endl;
	parameter_file << get_Wr()  << std::endl;
	parameter_file << get_Wu0() << std::endl;
	parameter_file << get_Wu1() << std::endl;
	parameter_file << get_Wu2() << std::endl;
	parameter_file << get_Wu3() << std::endl;
	parameter_file << get_Wt() << std::endl;
	
	// done writing parameteres, closing file
	parameter_file.close();

	std::cout << std::endl << std::endl << "Objective value" << std::endl;
	std::cout << "f(x*) = " << obj_value << std::endl;
/*
	std::cout << std::endl << "Final value of the constraints:" << std::endl;
	for (Index i=0; i<m ;i++) {
		std::cout << "g(" << i << ") = " << g[i] << std::endl;
	}
*/ 
  
}