/*
   MAIN file for path planning framework
   Set ipopt options here
   The object (class) holding the NLP inf, short_NLP is sent to the solving method.
   Some problem parameters are also defined with the methods in parameters.c

   Lars Grimsland - spring 2012
*/

#include "IpIpoptApplication.hpp"
#include "short_nlp.hpp"
#include "parameters.h"

#define PAUSE {printf("Press Enter to continue...\n"); fflush(stdin); getchar(); fflush(stdin);}

#include <iostream>

using namespace Ipopt;


int main(int argv, char* argc[])
{
  // Create a new instance of your nlp
  //  (use a SmartPtr, not raw)
  SmartPtr<TNLP> mynlp = new short_NLP();
  std::cout << "DERP ..... step: 1 \n";
  
  // Create a new instance of IpoptApplication
  //  (use a SmartPtr, not raw)
  // We are using the factory, since this allows us to compile this
  // example with an Ipopt Windows DLL
  SmartPtr<IpoptApplication> app = IpoptApplicationFactory();
  std::cout << "DERP ..... step: 2 \n";
  
  // Change some options
  app->Options()->SetStringValue("mu_strategy", "adaptive");
  app->Options()->SetStringValue("output_file", "ipopt.out");
  // app->Options()->SetStringValue("print_options_documentation", "yes"); // FUNKER IKKE ???????

  /* Exact Hessian vs BFGS */
  //app->Options()->SetStringValue("hessian_approximation", "exact"); // DEFAULT 
  //app->Options()->SetStringValue("hessian_approximation", "limited-memory");
  //app->Options()->SetStringValue("limited_memory_update_type", "bfgs");

  /* Derivative Checker */
  //app->Options()->SetStringValue("check_derivatives_for_naninf", "yes");
  //app->Options()->SetStringValue("derivative_test", "second-order"); //second-order, first-order
  //app->Options()->SetStringValue("derivative_test_print_all","yes");
  //app->Options()->SetNumericValue("point_perturbation_radius", 0.0); // max radius of random diversion from given initial point, default = 10;
  //app->Options()->SetNumericValue("derivative_test_perturbation", 1.0E-7); // perturbation in derivative test, Default 1.0E-8


  /* Evaluations/step lengths/Termination */
  //app->Options()->SetIntegerValue("max_iter", 7500); // Max iterations (default 3k)
 //app->Options()->SetNumericValue("acceptable_tol", 1.0E-4); // acceptable convergence tolerance, relative. Default 1.0E-6


 
  // Intialize the IpoptApplication and process the options
  ApplicationReturnStatus status;
  status = app->Initialize();
  std::cout << "DERP ..... step: 4 \n";
  if (status != Solve_Succeeded) {
    std::cout << std::endl << std::endl << "*** Error during initialization!" << std::endl;
    return (int) status;
  }
  std::cout << "DERP ..... step: 5 \n";
  // Ask Ipopt to solve the problem
  status = app->OptimizeTNLP(mynlp);
  
  std::cout << "DERP ..... step: 6 \n";
  if (status == Solve_Succeeded) {
    std::cout << std::endl << std::endl << "*** The problem solved!" << std::endl;
  }
  else {
    std::cout << std::endl << std::endl << "*** The problem FAILED!" << std::endl;
  }

  // As the SmartPtrs go out of scope, the reference count
  // will be decremented and the objects will automatically
  // be deleted.

	PAUSE

  return (int) status;
}
